"""
Oracle database async adaptor.

Starts separate threads for each connection, put them to the queue to offer pooling capabilities.
Clients obtain free "connection" (whats thread actually) from that queue. When the connection
is taken from the queue, logical transaction is started (means there is no command being
sent to the database to start the transaction). OracleThread internal state changes to "in transaction",
after that client is allowed to execute DBAPI commands like fetch/scalar/execute/etc.
Besides, when the transaction is started, ping command is sent to the database to perform
pessimistic check of the connection.
Every command from the client (which belongs to the main application thread) is passed to the thread
though it's AsyncThreadPipe (wrapper around queue that converts commands to Message objects and setup
futures for responses). Thread listen to the pipe and perform appropriate handling based on its internal
state (either "in transaction" or "ready").

NOTE: Doesn't support cancellation.

References:
https://oracle.github.io/node-oracledb/doc/api.html#dbcalltimeouts
https://cx-oracle.readthedocs.io/en/8.2.1/api_manual/connection.html#Connection.call_timeout
https://cx-oracle.readthedocs.io/en/latest/user_guide/connection_handling.html#easy-connect-syntax-for-connection-strings
https://docs.sqlalchemy.org/en/14/core/pooling.html#disconnect-handling-pessimistic
https://docs.sqlalchemy.org/en/14/core/pooling.html#reset-on-return
"""

import asyncio
from asyncio import AbstractEventLoop, Future, Queue as AQueue
from collections.abc import AsyncGenerator
from contextlib import asynccontextmanager
from enum import Enum
from queue import Queue
from threading import Thread, local
from typing import Any, Awaitable, Callable, Dict, List, Optional, Tuple, Union

from sqlalchemy import Sequence, sql
from sqlalchemy.engine import Connection, create_engine
from sqlalchemy.exc import DatabaseError
from sqlalchemy.pool import SingletonThreadPool

from ..errors.errors import Error
from ..errors.exception import VLException
from ..utils.log import Logger, logger as globalLogger
from .connection import AbstractDBConnection
from .pool import AbstractSessionPool


class Command(Enum):
    """
    Enum for oracle commands
    """

    begin = "begin"
    cursor = "cursor"
    fetchmany = "fetchmany"
    fetchall = "fetchall"
    fetchone = "fetchone"
    scalar = "scalar"
    execute = "execute"
    nextVal = "nextVal"
    commit = "commit"
    rollback = "rollback"
    endConnection = "end_connection"


class StopThread(Exception):
    """Raised on Command.endConnection."""


class Message:
    """
    Connector between oracle connection's threads and the main event loop.

    Attributes:
        command (Command): oracle command
        future (Future): asyncio future
        value (Optional[Any]): optional value
        mainLoop (AbstractEventLoop): event loop associated with the future
    """

    __slots__ = ("command", "value", "future", "mainLoop")

    def __init__(self, command: Command, future: Future, value: Any, mainLoop: AbstractEventLoop):
        """
        Args:
            command: oracle command
            future: asyncio future
            value: optional value
            mainLoop: event loop associated with the future
        """
        self.command: Command = command
        self.value = value
        self.future: Future = future
        self.mainLoop = mainLoop

    def __str__(self) -> str:
        return f"<Message: {self.command}, {self.value}>"

    def _addResult(self, data: Any):
        """Callback for resolving message feature."""
        if self.future.done():
            return

        if isinstance(data, Exception):
            self.future.set_exception(data)
        else:
            self.future.set_result(data)

    def reply(self, data: Any):
        """
        Pass the data to the main thread running the main event loop.

        Args:
            data: arbitary object
        """
        self.mainLoop.call_soon_threadsafe(self._addResult, data)


class AsyncThreadPipe:
    """
    Async thread pipe for work with messages
    Attributes:
        _queue (Queue): thread-safe queue for communication between main thread and oracle thread
        mainLoop (AbstractEventLoop): used event loop
    """

    def __init__(self):
        self._queue: "Queue[Message]" = Queue()
        self.mainLoop: AbstractEventLoop = asyncio.get_running_loop()

    def clientRequest(self, command: Command, value: Optional[Any] = None) -> Awaitable[Any]:
        """
        Put new command to pipe and return associated future.

        Args:
            command: command
            value: value if it needs
        Returns:
            future
        """
        future = self.mainLoop.create_future()
        message = Message(command, future, value, self.mainLoop)
        self._queue.put_nowait(message)
        return future


class OracleThread(Thread):
    """
    Oracle thread for use in session pool
    Attributes:
        name (str): thread name
        inputQueue (AsyncThreadPipe): async thread pipe to work with
        mainEventLoop (AbstractEventLoop): main event loop to work with
        logger (Logger): logger
        local (local): local storage for the thread
        local.engine (Engine): database engine
        connectTimeout: A float, seconds, connect timeout (used for connection checking as well)
        connectKwargs: A dict, kwargs for cx_Oracle.connect
        transaction: sqlalchemy obj representing transaction
    """

    def __init__(self, name: str, connectKwargs: dict, connectTimeout: float):
        super().__init__()
        self.name = name
        self.inputQueue = AsyncThreadPipe()
        self.mainEventLoop = asyncio.get_running_loop()
        self.local: local = local()
        self.logger = Logger(template=f"ora-{name}")
        self._connection: Optional[Connection] = None
        self.connectTimeout = connectTimeout
        self.connectKwargs = connectKwargs
        self.transaction = None

    @property
    def connection(self) -> Connection:
        """
        Get connection and initialize if it needed
        Returns:
            connection
        """

        if self._connection is None:
            self.logger.info("(Re)initialize connection")
            self._connection = self.local.engine.connect()
        return self._connection

    def _executeSqlCommand(self, message: Message) -> Any:
        """
        Execute sql command
        Args:
            message: message with command

        Returns:
            True if command successfully executed otherwise False
        """
        resProxy = self.connection.execute(message.value)
        if message.command == Command.cursor:
            result = resProxy.cursor
        elif message.command == Command.fetchall:
            result = resProxy.fetchall()
        elif message.command == Command.fetchone:
            result = resProxy.fetchone()
        elif message.command == Command.execute:
            result = resProxy.rowcount
        elif message.command == Command.scalar:
            result = resProxy.scalar()
        elif message.command == Command.nextVal:
            result = resProxy
        else:
            raise ValueError(f"bad input command {message.command}, transaction has already begun")
        return result

    def handleMessage(self, message: Message) -> Callable:
        """
        Handle incoming message while no transaction is running.
        Perform state transition if needed (by returning handler for next message).

        Raises:
            StopThread in case of rqeuest to end the connection

        Returns:
            Handler according to next state.
        """
        try:  # pylint: disable-msg=no-else-return
            if message.command == Command.endConnection:
                if self._connection:
                    self.connection.close()
                message.reply(True)
                raise StopThread

            if message.command == Command.rollback:
                message.reply(True)
                # Can get there due to disconnect
                return self.handleMessage

            if message.command != Command.begin:
                raise RuntimeError(f"bad input command {message.command}, transaction did not start")

            self.ping()
            self.transaction = self.connection.begin()
            message.reply(True)
        except DatabaseError as exc:
            message.reply(exc)
            return self.handleMessage

        return self.handleMessageInTransaction

    def handleMessageInTransaction(self, message: Message) -> Callable:
        """
        Handle incoming message while in transaction.
        Perform state transition if needed (by returning handler for next message).

        Raises:
            StopThread in case of rqeuest to end the connection

        Returns:
            Handler according to next state.
        """
        if message.command == Command.endConnection:
            self.transaction.close()
            message.reply(True)
            raise StopThread

        if message.command == Command.commit:
            self.transaction.commit()
            self.transaction = None
            message.reply(True)
            return self.handleMessage

        if message.command == Command.rollback:
            self.transaction.rollback()
            self.transaction = None
            message.reply(True)
            return self.handleMessage

        try:
            if message.command in (
                Command.cursor,
                Command.fetchall,
                Command.scalar,
                Command.nextVal,
                Command.execute,
                Command.fetchone,
            ):
                result = self._executeSqlCommand(message)
            elif message.command == Command.fetchmany:
                cursor, batchSize = message.value
                result = cursor.fetchmany(batchSize)
            else:
                raise RuntimeError("Command unsupported")
            message.reply(result)
        except DatabaseError as exc:
            message.reply(exc)

        return self.handleMessageInTransaction

    def _processingLoop(self) -> None:
        """
        Infinitely start and complete transactions.
        Must not raise anything.
        """
        handleMessage = self.handleMessage
        while True:
            message = self.inputQueue._queue.get()
            try:
                handleMessage = handleMessage(message)
            except StopThread:
                break
            except Exception as exc:  # pylint: disable-msg=broad-except
                message.reply(exc)
                self.logger.error(f"Unexpected error, exiting: {exc}")
                self.logger.exception(exc)
                break
            finally:
                self.inputQueue._queue.task_done()

    def run(self):
        """
        Run thread.

        Creates engine with a pool with 1 connection (we don't need any additional pool handling).
        """
        import cx_Oracle  # pylint: disable-msg=import-outside-toplevel

        self.local.engine = create_engine(
            "oracle://",
            creator=lambda: cx_Oracle.connect(**self.connectKwargs),
            poolclass=SingletonThreadPool,
            pool_size=1,
        )
        self._processingLoop()  # infinite

        self.logger.info("closing thread engine")
        self.local.engine.dispose()
        self.logger.info("thread engine closed")

    def ping(self):
        """
        Ensure database connection is open and working by doing round trip to the database.

        Raises:
            DatabaseError in case of exception during ping()
        """
        timeout = int(self.connectTimeout * 1000 / 2)  # timeout in oracle became x2 because it tries IPv4 then IPv6
        cxConn = self.connection.connection.connection  # connect timeout from the driver raised here
        cxConn.call_timeout = timeout
        try:
            cxConn.ping()
        except Exception as exc:
            # Can't recover from this error => close connection

            # Notify sqlalchemy Pool that current connection is invalid
            self._connection.invalidate()

            # Pretend that this is an sqlalchemy error
            # Thats because when we do connection.connect() we receive DatabaseError on timeout
            # and both exceptions are being handlend in the one place - connection context manager
            raise DatabaseError(  # pylint: disable-msg=raise-missing-from
                f"(cx_Oracle.DatabaseError) {exc}", orig=exc, params=None
            )

        cxConn.call_timeout = 0

    def execute(self, command: Command, value: Optional[Any] = None) -> Awaitable[Any]:
        """
        Put command to queue with optional value
        Args:
            command: command to execute
            value: optional value for command
        Returns:
            future
        """
        res = self.inputQueue.clientRequest(command, value)
        return res


class OracleSessionPool(AbstractSessionPool):
    """
    Connection for oracle

    Attributes:
        pool (Dict[str, OracleThread]): connection pool
        _nonActiveConnections (asyncio.Queue): async queue with non-active connections
        connectTimeout (int): connect timeout
    """

    def __init__(self, dsn, dblogin: str, dbpassword: str, connectTimeout: int):
        super().__init__(dsn, dblogin, dbpassword)
        self.pool: Dict[str, OracleThread] = {}
        self._nonActiveConnections: Optional[AQueue[OracleThread]] = None
        self.connectTimeout = connectTimeout
        self.connectKwargs = {
            "user": dblogin,
            "password": dbpassword,
            "dsn": f"{dsn}{'&' if '?' in dsn else '?'}connect_timeout={connectTimeout}",
        }

    async def initialize(self, sessionCount: int, **kwargs) -> None:
        """
        Initialize connection engine
        Args:
            sessionCount: session count
        """
        self.pool: Dict[str, OracleThread] = {
            str(sessionIndex): OracleThread(str(sessionIndex), self.connectKwargs, connectTimeout=self.connectTimeout)
            for sessionIndex in range(sessionCount)
        }
        self._nonActiveConnections = AQueue(maxsize=sessionCount)
        for thread in self.pool.values():
            thread.start()
            self._nonActiveConnections.put_nowait(thread)

    async def _getNonActiveConnection(self) -> OracleThread:
        """
        Wait for non active connection and return it
        Returns:
            non active connection
        """
        thread: OracleThread = await self._nonActiveConnections.get()
        if not thread.is_alive():
            globalLogger.warning("oracle thread is dead, recreating")
            thread = OracleThread(thread.name, self.connectKwargs, connectTimeout=self.connectTimeout)
            self.pool[thread.name] = thread
            thread.start()
        return thread

    async def releaseConnection(self, thread: OracleThread, logger: Logger):
        """
        Return connection to connection pool.

        Args:
            thread: current user connection
            logger: logger associated with connection

        """
        try:
            if not thread.is_alive():
                logger.warning("oracle thread is dead")
            else:
                """
                There is no guarantee that thread normal working. Thread is needing a some time for a correct stop after
                last command executing.
                """
                attempt = 1
                while True:
                    _, pending = await asyncio.wait({thread.execute(Command.rollback)}, timeout=5)
                    if pending:
                        """
                        We do not know why task is not ended and hope that the task will be done or thread will be
                        dead some time.
                        """
                        if not thread.is_alive():
                            logger.warning("oracle thread is dead")
                            break
                        logger.warning(f"oracle thread can not be released, attempt {attempt}")
                        attempt += 1
                    else:
                        break
        finally:
            # if thread is not allive, recreate connection in _getNonActiveConnection
            logger.debug("put connection to pool non active")
            self._nonActiveConnections.put_nowait(thread)

    @asynccontextmanager
    async def connection(self, logger: Logger) -> OracleThread:
        """
        Get connection to oracle
        Args:
            logger: logger
        """
        # Obtain free connection from the pool
        try:
            async with asyncio.timeout(self.connectTimeout):
                thread: OracleThread = await self._getNonActiveConnection()
        except asyncio.TimeoutError as ex:
            logger.error("ora connection pool: run out of connections")
            raise VLException(Error.DataBaseConnectionTimeoutError, 500, False) from ex

        # connect / begin transaction
        try:
            await thread.execute(Command.begin)

            yield thread

            await thread.execute(Command.commit)
        except DatabaseError as exc:
            if exc.args[0] == "(cx_Oracle.DatabaseError) DPI-1080: connection was closed by ORA-3156":
                self.printConnectionErrorToLog(exc, logger)
                raise VLException(Error.DataBaseConnectionTimeoutError, 500, False) from None
            if exc.args[0] == "(cx_Oracle.DatabaseError) ORA-12170: TNS:Connect timeout occurred":
                self.printConnectionErrorToLog(exc, logger)
                raise VLException(Error.DataBaseConnectionTimeoutError, 500, False) from None
            if exc.args[0] == "(cx_Oracle.DatabaseError) DPI-1080: connection was closed by ORA-3113":
                logger.error("connection was closed")
                raise VLException(Error.OtherDatabaseError.format("connection was closed"), 500, False) from None
            raise
        finally:
            # Eventually connection will be put back to the pool
            asyncio.create_task(self.releaseConnection(thread, logger))

    async def close(self) -> None:
        """
        Attempt to gracefully close all connection to db.
        """
        for thread in self.pool.values():
            if thread.is_alive():
                await thread.execute(Command.endConnection)

        for thread in self.pool.values():
            retryCount = 3
            while thread.is_alive() and retryCount:
                thread.join(self.connectTimeout)
                retryCount -= 1

            if thread.is_alive():
                globalLogger.warning(f"Tread {thread.get_indent()} has't stopped")


class OracleDBConnection(AbstractDBConnection):
    """
    Wrapper for a oracle db connection
    """

    def __init__(self, connection: OracleThread):
        """
        Initialize connection
        Args:
            connection: session pool for oracle
        """
        super().__init__(connection)

    async def asyncExecuteQuery(self, command: Command, *args) -> Any:
        """
        Execute a sql request in a thread
        Args:
            *args args of function

        Returns:
            result of sqlFunc
        """
        return await self.connection.execute(command, *args)

    async def execute(self, statement: sql.ClauseElement, data: Optional[Any] = None) -> int:
        """
        Description see :func:`~AbstractDBConnection.execute`.
        """
        return await self.asyncExecuteQuery(Command.execute, statement)

    async def fetchone(self, statement: sql.ClauseElement) -> Union[Tuple, None]:
        """
        Description see :func:`~AbstractDBConnection.fetchone`.
        """
        return await self.asyncExecuteQuery(Command.fetchone, statement)

    async def scalar(self, statement: sql.ClauseElement) -> Union[int, str, float, bytes, None]:
        """
        Description see :func:`~AbstractDBConnection.scalar`.
        """
        return await self.asyncExecuteQuery(Command.scalar, statement)

    async def nextVal(self, sequence: Sequence) -> int:
        """
        Description see :func:`~AbstractDBConnection.nextVal`.
        """
        return await self.asyncExecuteQuery(Command.nextVal, sequence)

    async def fetchall(self, statement: sql.ClauseElement) -> List[Tuple]:
        """
        Description see :func:`~AbstractDBConnection.fetchall`.
        """
        return await self.asyncExecuteQuery(Command.fetchall, statement)

    async def stream(self, statement: sql.ClauseElement, batchSize: int = 10000) -> AsyncGenerator[list[tuple], None]:
        """
        Description see :func:`~AbstractDBConnection.stream`.
        """
        with await self.asyncExecuteQuery(Command.cursor, statement) as cursor:
            while True:
                rows = await self.asyncExecuteQuery(Command.fetchmany, (cursor, batchSize))
                if not rows:
                    break
                yield rows
